/*
	MIT License http://www.opensource.org/licenses/mit-license.php
	Author Tobias Koppers @sokra
*/
"use strict";

const Module = require("./Module");
const Template = require("./Template");
const RawSource = require("webpack-sources").RawSource;

class MultiModule extends Module {

	constructor(context, dependencies, name) {
		super("javascript/dynamic");

		// Info from Factory
		this.context = context;
		this.dependencies = dependencies;
		this.name = name;
	}

	identifier() {
		return `multi ${this.dependencies.map((d) => d.request).join(" ")}`;
	}

	readableIdentifier(requestShortener) {
		return `multi ${this.dependencies.map((d) => requestShortener.shorten(d.request)).join(" ")}`;
	}

	build(options, compilation, resolver, fs, callback) {
		this.built = true;
		return callback();
	}

	needRebuild() {
		return false;
	}

	size() {
		return 16 + this.dependencies.length * 12;
	}

	updateHash(hash) {
		hash.update("multi module");
		hash.update(this.name || "");
		super.updateHash(hash);
	}

	source(dependencyTemplates, outputOptions) {
		const str = [];
		this.dependencies.forEach((dep, idx) => {
			if(dep.module) {
				if(idx === this.dependencies.length - 1)
					str.push("module.exports = ");
				str.push("__webpack_require__(");
				if(outputOptions.pathinfo)
					str.push(Template.toComment(dep.request));
				str.push(`${JSON.stringify(dep.module.id)}`);
				str.push(")");
			} else {
				// TODO replace with WebpackMissingModule module
				str.push("(function webpackMissingModule() { throw new Error(");
				str.push(JSON.stringify(`Cannot find module "${dep.request}"`));
				str.push("); }())");
			}
			str.push(";\n");
		});
		return new RawSource(str.join(""));
	}
}

module.exports = MultiModule;
